// Copyright 2018 The Flutter team. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_apm/apm_module.dart';
import 'package:flutter_apm/apm_ui_services_user_module.dart';
import 'package:flutter_apm/apm_services_gift_module.dart';
import 'package:flutter_apm/apm_user_properties_utils_module.dart';
import 'package:flutter_apm/apm_stat_utils_module.dart';
import 'package:flutter_apm/apm_config_module.dart';
import 'package:flutter_apm/apm_deeplink_utils_module.dart';
import 'package:flutter_apm_example/configurations/constants.dart';

class APMUtils {
  static void initSDKAPM() {
    beginMethod("initSDKAPM");

    APMModule.setServicesUserListener(null);
    APMModule.setServicesUserListener((event) {
      Map mapEvent = event;
      String method = mapEvent["method"];
      if(method == "apmServicesUserChanged") {
        log('method : apmServicesUserChanged');
        Map? user = mapEvent["user"];
        log('user : $user');
      }
      else if(method == "apmServicesUserBalanceChanged") {
        log('method : apmServicesUserBalanceChanged');
        Map? user = mapEvent["user"];
        log('user : $user');
      }
      else if(method == "apmServicesUserUnsubscribed") {
        log('method : apmServicesUserUnsubscribed');
      }
    });

    APMModule.initAppWithAppID(Constants.APM_PARTNER_ID, Constants.APM_PARTNER_SECRET, Constants.APP_ID);
    APMModule.setDebugMode(true);
    APMModule.setBaseUrl(Constants.APM_API_URL);

    APMModule.setOnBadgeClickListener(null);
    APMModule.setOnBadgeClickListener((event) {
      Map mapEvent = event;
      String method = mapEvent["method"];
      if(method == "onBadgeClick") {
        log('method : onBadgeClick');
        String? deeplink = mapEvent["deeplink"];
        log('deeplink : $deeplink');
      }
    });

    deeplinkUtilsModuleAddListener();

    endMethod("initSDKAPM");
  }

  static Future apmConnection(bool fail) async {
    String? firstname = 'Vincent';
    String? lastname = 'Ducastel';
    String? email = 'demo-awesome-ios@demo.com';
    String? facebookId = null;
    String? partnerClientId = 'demo-awesome-ios';
    String? oldClientId = null;
    Map? segments = null;
    var optIn = 0;

    if (fail) {
      partnerClientId = null;
    }

    try {
      var user = await APMUIServicesUserModule.userClientConnect(firstname, lastname, email, facebookId, partnerClientId, oldClientId, segments, optIn);
      log('APMUIServicesUserModule.userClientConnect() : $user');
    } catch (e) {
      log('APMUIServicesUserModule.userClientConnect() Error : $e');
    }
  }

  static void userLogout() async {
    try {
      await APMUIServicesUserModule.userLogout();
      log('Success : user logout !');

    } on Exception catch (e) {
      log('Error : user not logout...');
      log('e : $e');
    }
  }

  static void triggerAction(String actionName) {
    APMModule.triggerAction(actionName);
  }

  static void beginMethod(String method) {
    log('-----------------------------> $method');
  }

  static void endMethod(String method) {
    log('<----------------------------- $method');
  }

  static void testAll() async {
    beginMethod("testAll");

    initSDKAPM();
    APMModule.setInterstitialEnabled(false);
    await apmConnection(false);

    await testAPMConfigModule();
    await testAPMModule();
    await testAPMUIServicesUserModule();
    await testAPMServicesGiftModule();
    testAPMDeeplinkUtils();
    await testAPMStatUtilsModule();
    testAPMUserPropertiesUtilsModule();
    APMModule.setInterstitialEnabled(true);

    endMethod("testAll");
  }

  static Future testAPMModule() async {
    beginMethod("testAPMModule");

    APMModule.initAppWithPartnerID(Constants.APM_PARTNER_ID, Constants.APM_PARTNER_SECRET);
    APMModule.initAppWithAppID(Constants.APM_PARTNER_ID, Constants.APM_PARTNER_SECRET, Constants.APP_ID);
    APMModule.refreshSDK(true);

    const listActionName = ['tag1', 'tag2', 'tag3'];
    const listProperties = [{"property1": "value1"}, {"property1": "value1"}, {"property1": "value1"}];

    //triggerActions
    APMModule.triggerActions(listActionName);

    //triggerActionsWithProperties
    APMModule.triggerActionsWithProperties(listActionName, listProperties);

    //triggerActionsWithCompletion
    try {
      var completion = await APMModule.triggerActionsWithCompletion(listActionName, listProperties);
      log('APMModule.triggerActionsWithCompletion() : $completion');
    } catch (e) {
      log('APMModule.triggerActionsWithCompletion() Error : $e');
    }

    //triggerActionsWithTags
    const tags = [
      {
        "tagId": "tag1",
        "properties": {"property1": "value1"},
        "forDataTransfer": false,
        "policy": -1,
        "fromGenerosityBuffer": false
      },
      {
        "tagId": "tag2",
        "properties": {"property2": "value2"},
        "forDataTransfer": false,
        "policy": -1,
        "fromGenerosityBuffer": false
      }
    ];
    try {
      var completion = await APMModule.triggerActionsWithTags(tags);
      log('APMModule.triggerActionsWithTags() : $completion');
    } catch (e) {
      log('APMModule.triggerActionsWithTags() Error : $e');
    }

    //triggerAction
    APMModule.triggerAction("demoAction");

    //triggerActionWithProperties
    Map properties = {
      "zipcode": "33000"
    };
    APMModule.triggerActionWithProperties("demoAction", properties);

    //triggerActionWithCompletion
    try {
      var completion = await APMModule.triggerActionWithCompletion("tag1");
      log('APMModule.triggerActionWithCompletion() : $completion');
    } on PlatformException catch (e) {
      log('APMModule.triggerActionWithCompletion() Error : $e');
    }

    //triggerActionWithPropertiesAndCompletion
    try {
      var completion = await APMModule.triggerActionWithPropertiesAndCompletion("tag1", properties);
      log('APMModule.triggerActionWithPropertiesAndCompletion() : $completion');
    } on PlatformException catch (e) {
      log('APMModule.triggerActionWithPropertiesAndCompletion() Error : $e');
    }

    //triggerActionWithTag
    const tag = {
      "tagId": "tag1",
      "properties": {"property1": "value1"},
      "forDataTransfer": false,
      "policy": -1,
      "fromGenerosityBuffer": false
    };
    try {
      var completion = await APMModule.triggerActionWithTag(tag);
      log('APMModule.triggerActionWithTag() : $completion');
    } on PlatformException catch (e) {
      log('APMModule.triggerActionWithTag() Error : $e');
    }

    APMModule.setBaseUrl(Constants.APM_API_URL);
    log('APMModule.getBaseUrl() : ${await APMModule.getBaseUrl()}');
    APMModule.setDebugMode(true);
    log('APMModule.isDebugMode() : ${await APMModule.isDebugMode()}');
    APMModule.setDeviceStatus(0);
    APMModule.setDeviceOptIn(0);

    APMModule.setInterstitialEnabled(true);

    APMModule.setBadgeEnabled(false);
    APMModule.setInterstitialEnabled(false);
    APMModule.setNotificationFullscreenEnabled(false);

    APMModule.setBadgeEnabled(true);
    APMModule.setNotificationFullscreenEnabled(true);

    Map user = await APMModule.user();
    log('APMModule.user() : $user');
    if(user != null)
    {
      Map userJourney = user["userJourney"];
      log('APMModule.user.userJourney : $userJourney');
      List statuses = userJourney["status"];
      Map status = statuses[0];
      log('APMModule.user.userJourney.status[0] : $status');
      log('APMModule.user.userJourney.status[0].label : ${status["label"]}');

      List? acquiredTags = user["acquiredTags"];
      if(acquiredTags != null)
      {
        log('APMModule.user.acquiredTags : $acquiredTags');
      }
    }

    Map device = await APMModule.device();
    log('APMModule.device() : $device');

    APMModule.hideBadge();
    await delay(2000);
    APMModule.showBadge();
    await delay(2000);
    log('APMModule.isOpenBigBadge() : ${await APMModule.isOpenBigBadge()}');
    await delay(2000);
    APMModule.openBigBadge();
    await delay(2000);
    log('APMModule.closeBigBadge() : ${await APMModule.closeBigBadge()}');
    await delay(2000);
    APMModule.showOnboarding();
    await delay(2000);
    APMModule.showOnboardingConnected(false);
    await delay(2000);
    APMModule.openBigBadgeWithController(await APMConfigModule.APM_SHOW_CONTROLLER_MY_ACCOUNT());
    await delay(2000);
    log('APMModule.closeBigBadge() : ${await APMModule.closeBigBadge()}');
    await delay(2000);

    endMethod("testAPMModule");
  }

  static Future testAPMConfigModule() async {
    beginMethod("testAPMConfigModule");

    log('APMConfigModule.APM_BADGE_NATURE_BANNER : ${await APMConfigModule.APM_BADGE_NATURE_BANNER()}');
    log('APMConfigModule.APM_BADGE_NATURE_STICKER : ${await APMConfigModule.APM_BADGE_NATURE_STICKER()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_DAILY_CHALLENGE : ${await APMConfigModule.APM_SHOW_CONTROLLER_DAILY_CHALLENGE()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_MY_ACCOUNT : ${await APMConfigModule.APM_SHOW_CONTROLLER_MY_ACCOUNT()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_EARNS : ${await APMConfigModule.APM_SHOW_CONTROLLER_EARNS()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_BURNS : ${await APMConfigModule.APM_SHOW_CONTROLLER_BURNS()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_GIFTS : ${await APMConfigModule.APM_SHOW_CONTROLLER_GIFTS()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_STATUS : ${await APMConfigModule.APM_SHOW_CONTROLLER_STATUS()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_PAGE : ${await APMConfigModule.APM_SHOW_CONTROLLER_PAGE()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_CHALLENGES : ${await APMConfigModule.APM_SHOW_CONTROLLER_CHALLENGES()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_TROPHIES : ${await APMConfigModule.APM_SHOW_CONTROLLER_TROPHIES()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_INFOS : ${await APMConfigModule.APM_SHOW_CONTROLLER_INFOS()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_GIFT : ${await APMConfigModule.APM_SHOW_CONTROLLER_GIFT()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_BARCODE : ${await APMConfigModule.APM_SHOW_CONTROLLER_BARCODE()}');
    log('APMConfigModule.APM_SHOW_CONTROLLER_UNSUBSCRIBE : ${await APMConfigModule.APM_SHOW_CONTROLLER_UNSUBSCRIBE()}');

    endMethod("testAPMConfigModule");
  }

  static Future testAPMUIServicesUserModule() async {
    beginMethod("testAPMUIServicesUserModule");

    try {
      Map<String, dynamic> userJson = <String, dynamic>{
        'partnerClientId': "demo-flutter",
        'email': "demo-flutter@demo.com",
        'optIn': 0,
      };
      Map? userJson2 = await APMUIServicesUserModule.userClientConnectWithJSON(userJson);
      log('APMUIServicesUserModule.userClientConnectWithJSON() : $userJson2');
    } on PlatformException catch (e) {
      log('APMUIServicesUserModule.userClientConnectWithJSON() Error : $e');
    }

    try {
      await APMUIServicesUserModule.userLogout();
      log('APMUIServicesUserModule.userLogout() OK');
    } catch (e) {
      log('APMUIServicesUserModule.userLogout() Error : $e');
    }

    //UserClientConnect
    await apmConnection(false);

    try {
      await APMUIServicesUserModule.userDelete();
      log('APMServicesUserModule.userDelete() OK');
    } catch (e) {
      log('APMServicesUserModule.userDelete() Error : $e');
    }

    endMethod("testAPMUIServicesUserModule");
  }

  static Future testAPMServicesGiftModule() async {
    beginMethod("testAPMServicesGiftModule");

    //Connection before giftList
    await apmConnection(false);

    try {
      var gifts = await APMServicesGiftModule.giftList("0", "0", 1);
      log('APMServicesGiftModule.giftList() : $gifts');
    } on Exception catch (e) {
      log('APMServicesGiftModule.giftList() Error : $e');
    }

    endMethod("testAPMServicesGiftModule");
  }

  static void testAPMDeeplinkUtils() {
    beginMethod("testAPMDeeplinkUtils");

    APMDeeplinkUtilsModule.removeListener();
    deeplinkUtilsModuleAddListener();

    endMethod("testAPMDeeplinkUtils");
  }

  static void testAPMUserPropertiesUtilsModule() {
    beginMethod("testAPMUserPropertiesUtilsModule");

    var properties = {
      "zipcode": "33000"
    };
    APMUserPropertiesUtilsModule.addUserProperties(properties);
    APMUserPropertiesUtilsModule.addUserProperty("33000", "zipcode");
    APMUserPropertiesUtilsModule.removeUserProperty("zipcode");
    APMUserPropertiesUtilsModule.removeUserProperties();

    endMethod("testAPMUserPropertiesUtilsModule");
  }

  static Future testAPMStatUtilsModule() async {
    beginMethod("testAPMStatUtilsModule");

    try {
      var optins = await APMStatUtilsModule.setEnabledOptinStat(true);
      log('APMStatUtilsModule.setEnabledOptinStat(true) : $optins');
    } catch (e) {
      log('APMStatUtilsModule.setEnabledOptinStat(true) Error : $e');
    }

    try {
      var optins = await APMStatUtilsModule.setEnabledOptinStat(false);
      log('APMStatUtilsModule.setEnabledOptinStat(false) : $optins');
    } catch (e) {
      log('APMStatUtilsModule.setEnabledOptinStat(false) Error : $e');
    }

    endMethod("testAPMStatUtilsModule");
  }

  static Future delay(int ms) async {
    await Future.delayed(Duration(seconds: ms ~/ 1000));
  }

  static void deeplinkUtilsModuleAddListener() {
    APMDeeplinkUtilsModule.addListener((event) {
      Map mapEvent = event;
      String method = mapEvent["method"];
      if(method == "apmDeeplinkOnboardingButtonOkClicked") {
        log('method : apmDeeplinkOnboardingButtonOkClicked');
        APMUtils.apmConnection(false);
      }
      else if(method == "apmDeeplinkOnboardingButtonLaterClicked") {
        log('method : apmDeeplinkOnboardingButtonLaterClicked');
      }
      else if(method == "apmDeeplinkOnboardingButtonNeverClicked") {
        log('method : apmDeeplinkOnboardingButtonNeverClicked');
      }
      else if(method == "apmDeeplinkActionButtonGoClicked") {
        String? url = mapEvent["url"];
        String? action = mapEvent["action"];
        Map? params = mapEvent["params"];
        log('method : apmDeeplinkActionButtonGoClicked');
        log('url : $url');
        log('action : $action');
        log('params : $params');
      }
      else if(method == "apmDeeplinkClicked") {
        String? url = mapEvent["url"];
        log('method : apmDeeplinkClicked');
        log('url : $url');
      }
    });
  }
}
